import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';
 
class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.domWebGL = document.createElement('div');
        document.body.appendChild(this.domWebGL);

        let scene = new THREE.Scene();

        let renderer = new THREE.WebGLRenderer( { antialias: true });
        renderer.autoClear = false;

        this.domWebGL.appendChild(renderer.domElement);

        this._FRUSTUM_SIZE = 600;

        this._stats = new Stats();
        this.domWebGL.appendChild(this._stats.dom);

        window.onresize = this.resize.bind(this);
        window.onkeydown = this.onKeyDown.bind(this);

        this.renderer = renderer;
        this.scene = scene;

        this._setupCamera();
        this._setupModel();
        //this._setupLights();
        //this._setupControl();

        

        this.resize();
    }

    _setupModel() {
        const mesh = new THREE.Mesh(
            new THREE.SphereGeometry(100, 16, 8),
            new THREE.MeshBasicMaterial({ color: 0xffffff, wireframe: true })
        );

        this.scene.add(mesh);

        const mesh2 = new THREE.Mesh(
            new THREE.SphereGeometry(50, 16, 8),
            new THREE.MeshBasicMaterial( { color: 0x00ff00, wireframe: true })
        );

        mesh2.position.y = 150;
        mesh.add(mesh2);

        const mesh3 = new THREE.Mesh(
            new THREE.SphereGeometry(5, 16, 8),
            new THREE.MeshBasicMaterial( { color: 0x0000ff, wireframe: true })
        );

        mesh3.position.z = 150;
        this._cameraRig.add(mesh3);

        const geometry = new THREE.BufferGeometry();
        const vertices = [];

        for(let i=0; i<10000; i++) {
            vertices.push(THREE.MathUtils.randFloatSpread(2000)); // x
            vertices.push(THREE.MathUtils.randFloatSpread(2000)); // y
            vertices.push(THREE.MathUtils.randFloatSpread(2000)); // z
        }

        geometry.setAttribute('position', new THREE.Float32BufferAttribute(vertices, 3));
        const particles = new THREE.Points(geometry, new THREE.PointsMaterial({ color: 0xffffff }));
        this.scene.add(particles);

        this._mesh = mesh;
    }

    /*
    _setupLights() {}
    */

    _setupCamera() {
        const aspect = window.innerWidth / window.innerHeight;
        const camera = new THREE.PerspectiveCamera(50, 0.5 * aspect, 1, 10000);
        camera.position.z = 2500;

        const cameraPerspective = new THREE.PerspectiveCamera(50, 0.5*aspect, 150, 1000);
        const cameraPerspectiveHelper = new THREE.CameraHelper(cameraPerspective);
        this.scene.add(cameraPerspectiveHelper);

        const cameraOrtho = new THREE.OrthographicCamera(0.5 * this._FRUSTUM_SIZE * aspect / - 2, 
            0.5 * this._FRUSTUM_SIZE * aspect / 2, 
            this._FRUSTUM_SIZE / 2, this._FRUSTUM_SIZE / - 2, 150, 1000 );
        const cameraOrthoHelper = new THREE.CameraHelper(cameraOrtho);
        this.scene.add(cameraOrthoHelper);

        cameraOrtho.rotation.y = Math.PI;
        cameraPerspective.rotation.y = Math.PI;

        const cameraRig = new THREE.Group();
        cameraRig.add(cameraOrtho);
        cameraRig.add(cameraPerspective);
        
        this.scene.add(cameraRig);

        this._camera = camera;
        this._activeCamera = cameraPerspective;
        this._activeHelper = cameraPerspectiveHelper;
        this._cameraPerspective = cameraPerspective;
        this._cameraPerspectiveHelper = cameraPerspectiveHelper;
        this._cameraOrtho = cameraOrtho;
        this._cameraOrthoHelper = cameraOrthoHelper;
        this._cameraRig = cameraRig;
    }

    update(time) {
        const r = time * 0.0005;

        this._mesh.position.x = 700 * Math.cos(r);
        this._mesh.position.z = 700 * Math.sin(r);
        this._mesh.position.y = 700 * Math.sin(r);

        this._mesh.children[0].position.x = 50 * Math.cos(2*r);
        this._mesh.children[0].position.z = 50 * Math.sin(r);

        if(this._activeCamera === this._cameraPerspective) {
            this._cameraPerspective.fov = 35 + 30 * Math.sin(0.5 * r);
            this._cameraPerspective.far = this._mesh.position.length();
            this._cameraPerspective.updateProjectionMatrix();

            this._cameraPerspectiveHelper.update();
            
            this._cameraPerspectiveHelper.visible = true;
            this._cameraOrthoHelper.visible = false;
        } else {
            this._cameraOrtho.far = this._mesh.position.length();
            this._cameraOrtho.updateProjectionMatrix();

            this._cameraOrthoHelper.update();
            
            this._cameraPerspectiveHelper.visible = false;
            this._cameraOrthoHelper.visible = true;
        }
    }

    render(time) {
        requestAnimationFrame(this.render.bind(this));

        this.update(time);
        
        this._cameraRig.lookAt(this._mesh.position);
        this.renderer.clear();

        this._activeHelper.visible = false;
        this.renderer.setViewport(0, 0, window.innerWidth/2, window.innerHeight);
        this.renderer.render(this.scene, this._activeCamera);
        this._activeHelper.visible = true;

        this.renderer.setViewport(window.innerWidth/2, 0, window.innerWidth/2, window.innerHeight);
        this.renderer.render(this.scene, this._camera);

        this._stats.update();
    }

    onKeyDown(event) {
        switch(event.keyCode) {
        case 79: // O
            this._activeCamera = this._cameraOrtho;
            this._activeHelper = this._cameraOrthoHelper;
            break;
        case 80: // P
            this._activeCamera = this._cameraPerspective;
            this._activeHelper = this._cameraPerspectiveHelper;
            break;
        }
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.setSize(window.innerWidth, window.innerHeight);

        this._camera.aspect = 0.5 * aspect;
        this._camera.updateProjectionMatrix();

        this._cameraPerspective.aspect = 0.5 * aspect;
        this._cameraPerspective.updateProjectionMatrix();

        this._cameraOrtho.left = -0.5 * this._FRUSTUM_SIZE * aspect / 2;
        this._cameraOrtho.right =  0.5 * this._FRUSTUM_SIZE * aspect / 2;
        this._cameraOrtho.top = this._FRUSTUM_SIZE / 2;
        this._cameraOrtho.bottom = -this._FRUSTUM_SIZE / 2;
        this._cameraOrtho.updateProjectionMatrix();
    }
}

window.onload = function () {
    (new App()).render();
}